﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Infragistics.Web.UI.GridControls;
using Infragistics.Web.UI.ListControls;

namespace TestAdminSQLTable
{
    public partial class Generic : System.Web.UI.Page
    {
        private string strTable;
        private SQLTable sqlTable;

        protected void Page_Load(object sender, EventArgs e)
        {
            //strTable = Request.QueryString["Table"];
            strTable = "User";
            sqlTable = SQLTable.GetAdministrableTables().Find(delegate(SQLTable t) { return t.TableName == strTable; });

            if (!IsPostBack)
                ConstructGrid();

            mainGrid.DataSource = GetDataSource();
            mainGrid.DataKeyFields = sqlTable.KeyFields;
            mainGrid.DataBind();
        }

        private SqlDataSource GetDataSource()
        {
            SqlDataSource sqlDS = new SqlDataSource();
            sqlDS.ConnectionString = ConfigurationManager.ConnectionStrings["MyConnectionString"].ConnectionString;
            sqlDS.SelectCommand = sqlTable.SelectCommand;
            if (!string.IsNullOrEmpty(sqlTable.SelectCommand) && !sqlTable.SelectCommand.ToUpper().StartsWith("SELECT"))
                sqlDS.SelectCommandType = SqlDataSourceCommandType.StoredProcedure;
            else
                sqlDS.SelectCommandType = SqlDataSourceCommandType.Text;
            sqlDS.SelectParameters.Clear();

            sqlDS.DeleteCommand = sqlTable.DeleteCommand;
            if (!string.IsNullOrEmpty(sqlTable.DeleteCommand) && !sqlTable.DeleteCommand.ToUpper().StartsWith("DELETE"))
                sqlDS.DeleteCommandType = SqlDataSourceCommandType.StoredProcedure;
            else
                sqlDS.DeleteCommandType = SqlDataSourceCommandType.Text;
            sqlDS.DeleteParameters.Clear();

            sqlDS.UpdateCommand = sqlTable.UpdateCommand;
            if (!string.IsNullOrEmpty(sqlTable.UpdateCommand) && !sqlTable.UpdateCommand.ToUpper().StartsWith("UPDATE"))
                sqlDS.UpdateCommandType = SqlDataSourceCommandType.StoredProcedure;
            else
                sqlDS.UpdateCommandType = SqlDataSourceCommandType.Text;
            sqlDS.UpdateParameters.Clear();

            sqlDS.InsertCommand = sqlTable.InsertCommand;
            if (!string.IsNullOrEmpty(sqlTable.InsertCommand) && !sqlTable.InsertCommand.ToUpper().StartsWith("INSERT"))
                sqlDS.InsertCommandType = SqlDataSourceCommandType.StoredProcedure;
            else
                sqlDS.InsertCommandType = SqlDataSourceCommandType.Text;
            sqlDS.InsertParameters.Clear();

            foreach (SQLColumn column in sqlTable.GetColumns())
            {
                if (column.DeleteParam)
                    sqlDS.DeleteParameters.Add(new Parameter(column.ColumnName));
                if (column.UpdateParam)
                    sqlDS.UpdateParameters.Add(new Parameter(column.ColumnName));
                if (column.InsertParam)
                    sqlDS.InsertParameters.Add(new Parameter(column.ColumnName));
            }
            return sqlDS;

        }

        private void ConstructGrid()
        {
            mainGrid.Behaviors.CreateBehavior<EditingCore>();
            mainGrid.Behaviors.EditingCore.Behaviors.CreateBehavior<CellEditing>();
            mainGrid.Behaviors.EditingCore.Behaviors.CreateBehavior<RowAdding>();
            mainGrid.Behaviors.EditingCore.Behaviors.CreateBehavior<RowDeleting>();

            mainGrid.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings.Clear();
            mainGrid.Behaviors.EditingCore.Behaviors.RowAdding.ColumnSettings.Clear();
            //mainGrid.Behaviors.Filtering.ColumnSettings.Clear();
            mainGrid.EditorProviders.Clear();

            bool bAllowUpdate = !string.IsNullOrEmpty(sqlTable.UpdateCommand);
            mainGrid.Behaviors.EditingCore.Behaviors.RowAdding.Enabled = !string.IsNullOrEmpty(sqlTable.InsertCommand);
            mainGrid.Behaviors.EditingCore.Behaviors.RowDeleting.Enabled = !string.IsNullOrEmpty(sqlTable.DeleteCommand);

            foreach (SQLColumn column in sqlTable.GetColumns())
            {
                if (column.FieldType == "Boolean")
                {
                    BoundCheckBoxField bdf = new BoundCheckBoxField();
                    bdf.Key = column.ColumnName;
                    bdf.Header.Text = column.DisplayColumnName;
                    bdf.DataFieldName = column.ColumnName;
                    bdf.Hidden = !column.Visible;
                    mainGrid.Columns.Add(bdf);
                }
                else
                {
                    BoundDataField bdf = new BoundDataField();
                    bdf.Key = column.ColumnName;
                    bdf.Header.Text = column.DisplayColumnName;
                    bdf.DataFieldName = column.ColumnName;
                    bdf.Hidden = !column.Visible;
                    bdf.HtmlEncode = false;
                    mainGrid.Columns.Add(bdf);
                }

                if (!string.IsNullOrEmpty(column.ForeignTable) && !string.IsNullOrEmpty(column.ForeignValueColumn))
                {
                    string strDataSourceQuery = "SELECT ";
                    if (!string.IsNullOrEmpty(column.ForeignTextColumn))
                        strDataSourceQuery += column.ForeignTextColumn + ", ";
                    strDataSourceQuery += column.ForeignValueColumn + " FROM " + column.ForeignTable;
                    SqlDataSource ds = new SqlDataSource(ConfigurationManager.ConnectionStrings["MyConnectionString"].ConnectionString, strDataSourceQuery);
                    ds.ID = column.ColumnName + "datasource";

                    DropDownProvider ddp = new DropDownProvider();
                    ddp.EditorControl.DataSource = ds;
                    ddp.EditorControl.DisplayMode = DropDownDisplayMode.DropDown;
                    ddp.EditorControl.DataKeyFields = column.ForeignValueColumn;
                    ddp.EditorControl.TextField = !string.IsNullOrEmpty(column.ForeignTextColumn) ? column.ForeignTextColumn : column.ForeignValueColumn;
                    ddp.EditorControl.ValueField = column.ForeignValueColumn;
                    ddp.EditorControl.DropDownItemBinding.TextField = !string.IsNullOrEmpty(column.ForeignTextColumn) ? column.ForeignTextColumn : column.ForeignValueColumn;
                    ddp.EditorControl.DropDownItemBinding.ValueField = column.ForeignValueColumn;
                    ddp.EditorControl.ID = column.ColumnName + "DropDownEditorControl";
                    ddp.ID = column.ColumnName + "DropDown";
                    mainGrid.EditorProviders.Add(ddp);

                    EditingColumnSetting csttg = new EditingColumnSetting(mainGrid);
                    csttg.ColumnKey = column.ColumnName;
                    csttg.EditorID = ddp.ID;
                    csttg.ReadOnly = column.ReadOnly || !bAllowUpdate;
                    mainGrid.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings.Add(csttg);

                    RowAddingColumnSetting Acsttg = new RowAddingColumnSetting(mainGrid);
                    Acsttg.ColumnKey = column.ColumnName;
                    Acsttg.EditorID = ddp.ID;
                    Acsttg.DefaultValue = 0;
                    mainGrid.Behaviors.EditingCore.Behaviors.RowAdding.ColumnSettings.Add(Acsttg);

                    //ColumnFilteringSetting Fcsttg = new ColumnFilteringSetting(mainGrid);
                    //Fcsttg.ColumnKey = column.ColumnName;
                    //Fcsttg.EditorID = ddp.ID;
                    //mainGrid.Behaviors.Filtering.ColumnSettings.Add(Fcsttg);
                }
                else
                {
                    if (column.ReadOnly || !bAllowUpdate)
                    {
                        EditingColumnSetting csttg = new EditingColumnSetting(mainGrid);
                        csttg.ColumnKey = column.ColumnName;
                        csttg.ReadOnly = true;
                        mainGrid.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings.Add(csttg);
                    }
                }


            }

        }



        protected void btnSave_Click(object sender, EventArgs e)
        {
            //MainDataSource.Update();
        }

        protected void mainGrid_RowDeleted(object sender, RowDeletedEventArgs e)
        {
            HandleError(e);
        }

        protected void mainGrid_RowAdded(object sender, RowAddedEventArgs e)
        {
            HandleError(e);
        }


        private void HandleError(EditedEventArgs e)
        {
            if (e.Exception != null)
            {
                e.ExceptionHandled = true;
                mainGrid.CustomAJAXResponse.Message = e.Exception.Message;
                mainGrid.CustomAJAXResponse.Tag = "Error";
            }
        }

        protected void mainGrid_RowUpdated(object sender, RowUpdatedEventArgs e)
        {
            HandleError(e);
        }
    }
}